/***********************************************************************/
/*Schule	: IBZ/Zti Elektrotechniker HF -> Elektronik   	       	   */		
/*Abschluss	: Diplomarbeit TEEL7				       				   */
/*Projekt	: Helmsteuerung fr ein Mikrokopter		       			   */
/*Programm	: PPM_Signal				       	       				   */
/*Ersteller	: Antonio Carlucci                                     	   */
/*Datum		: 08.09.2007					       					   */
/***********************************************************************/


#include <T89C51AC2.h>

void init_adc(void);						/* Prototyp fr ADC initialisieren */
void init_timer0(void);						/* Prototyp fr Timer 0 initialisieren */
void pulse_pause(void);						/* Prototyp fr Pulse-Pause von 0.3ms auszugeben, bzw. 
								   			   um Register von Timer 0 zu laden */
void warte_ueberlauf(void);					/* Prototyp fr Warteschlaufe bis berlauf stattfindet */
void wait_speaker(void);					/* Prototyp fr Warteschlaufe Speaker und rote LED */


void analog_kanal_einlesen(void);
unsigned int pulse_auswertung_kanal_0_1_3(unsigned char analog_kanal[], unsigned char kanal_durchlauf);
unsigned int pulse_auswertung_kanal_2(unsigned char analog_kanal[], unsigned char kanal_durchlauf);
unsigned int pulse_auswertung_kanal_4(unsigned char analog_kanal[], unsigned char kanal_durchlauf);
unsigned int pulse_auswertung_kanal_5(void);
unsigned int pulse_auswertung_kanal_6(void);
unsigned int pulse_auswertung_kanal_7(void);
void start_pulse_auswertung(unsigned int kanal_pulse[], unsigned char anz_kanaele);
void batterie_ueberwachung(unsigned char[], unsigned char batterie_index);



unsigned char analog_kanal[6]= {0x00}; 		/* Analogwert Kanal 0 bis 5 */
unsigned char umgewandelter_wert= 0x00;		/* Umgewandelter Analogwert */
bit ende_umwandlung= 0;           			/* Software flag 		*/

bit flag_reload= 0;                         /* Flag fr den Timer 0 n-Mal zu reloaden */
bit flag_ueberlauf= 0;						/* Flag fr berlauf Timer 0 */
unsigned char timer_durchlauf;
unsigned char t= 0;                         


int main(void)
{
    unsigned char kanal_durchlauf;
    unsigned int kanal_pulse[8];
    unsigned char batterie_index = 5;
    
    init_adc();								/* ADC initialisieren */
    init_timer0();							/* Timer 0 initialisieren  */
	
	P0= 0x20;								/* Grne Betriebs LED auf Pin P0.5 einschalten */
    P2_1= 1;                                /* Pin P2_1 als Input definieren */
    P2_2= 1;                                /* Pin P2_2 als Input definieren */
    P2_3= 1;                                /* Pin P2_3 als Input definieren */
		
	while(1)								/* Endlosschleife um 8 Kanal PPM-Signal auszugeben */
	{
	    analog_kanal_einlesen();
        
        P2_0= 1;								 /* Port P2.0 auf High setzen */
        
        for(kanal_durchlauf=0; kanal_durchlauf<=9; kanal_durchlauf++)						 /* Schleife fr 9 Durchlufe  */					
        {    
            switch(kanal_durchlauf)
            {
                case 0:     kanal_pulse[0]= pulse_auswertung_kanal_0_1_3(analog_kanal, kanal_durchlauf);
                            break;
                            
                case 1:     kanal_pulse[1]= pulse_auswertung_kanal_0_1_3(analog_kanal, kanal_durchlauf);
                            break;
                            
                case 2:     kanal_pulse[2]= pulse_auswertung_kanal_2(analog_kanal, kanal_durchlauf); 
                            break;
                          
                case 3:     kanal_pulse[3]= pulse_auswertung_kanal_0_1_3(analog_kanal, kanal_durchlauf);
                            break;
                            
                case 4:     kanal_pulse[4]= pulse_auswertung_kanal_4(analog_kanal, kanal_durchlauf);
                            break;
                            
                case 5:     kanal_pulse[5]= pulse_auswertung_kanal_5();
                            break;        

                case 6:     kanal_pulse[6]= pulse_auswertung_kanal_6();
                            break; 

                case 7:     kanal_pulse[7]= pulse_auswertung_kanal_7();
                            break; 

                case 8:     start_pulse_auswertung(kanal_pulse, 8);
                            break;

                case 9:     batterie_ueberwachung(analog_kanal, batterie_index);
                            break;
            }   
        }
    }        
}        
            
            

/***********************************************************************/
/* Funktion			: analog_kanal_einlesen                        	   */
/* Funktion Input	: void					                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

void analog_kanal_einlesen(void)
{
    unsigned char kanalwahl;					/* Kanalwahl in der for-Schleife*/
    unsigned char i;

    for(i=0x00; i<=0x05; i++)				    /* Schleife fr Analogkanle 0-5 einlesen */
    {
        kanalwahl= i;
		ADCON &= ~0x07;                  	    /* Alle Kanle lschen Feld ADCON[2:0] */
   	    ADCON |= kanalwahl;                	    /* Auswahl Kanal 0 bis 5   */
   		ADCON &= ~0x40;                  	    /* Standard Modus -> 8 bit */
   		ADCON |= 0x08;                   	    /* Umwandlung starten	   */

   		while(!ende_umwandlung);       		    /* warten auf Ende Umwandlung */
   		ende_umwandlung=0;             		    /* Software flag lschen   */
   		analog_kanal[i]= umgewandelter_wert;    /* Wert in entsp. Variable speichern  */
    }
    ADCON &= ~0x08;                             /* Umwandlung stoppen */    
}


/***********************************************************************/
/* Funktion			: batterie_ueberwachung                        	   */
/* Funktion Input	: unsigned char			                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

void batterie_ueberwachung(unsigned char analog_kanal[], unsigned char batterie_index)
{                             
    if(analog_kanal[batterie_index] < 0xBE )				/* Wenn Analog Kanal 5 (Vref Speaker) kleiner 1.75 V, dann Speaker und rote LED ein */
    {
        wait_speaker();						                /* Warteschlaufe Speaker -> entsprechende Frequenz am Speaker kreieren */
		wait_speaker();						                /* Warteschlaufe Speaker -> entsprechende Frequenz am Speaker kreieren */
    }
}


/***********************************************************************/
/* Funktion			: pulse_auswertung_kanal_0_1_3               	   */
/* Funktion Input	: unsigned char	     	                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

unsigned int pulse_auswertung_kanal_0_1_3(unsigned char analog_kanal[], unsigned char kanal_durchlauf)
{
    unsigned int kanal_pulse1[11];
    
    if((analog_kanal[kanal_durchlauf] <= 0x71))	                                            /* Wert kleiner gleich 1.04 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x12; 						                    /* Timer 0 Register fr berlauf nach 0.7ms laden */			 
	    TH0= 0xF5;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[0]= 0x0F9C;		                /* Kanal-Pulse mit 1.0 ms fr Start-Pulse-Berechnung laden */
    }

    else if((analog_kanal[kanal_durchlauf] >= 0x72 ) && (analog_kanal[kanal_durchlauf] <= 0x7C))    /* Wert zwischen 1.05 V und 1.14 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x82; 						                    /* Timer 0 Register fr berlauf nach 0.8ms laden */			 
	    TH0= 0xF3;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();				                        /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	   	warte_ueberlauf();			                        /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[1]= 0x1122;		                /* Kanal-Pulse mit 1.1 ms fr Start-Pulse-Berechnung laden */
    }

	else if((analog_kanal[kanal_durchlauf] >= 0x7D ) && (analog_kanal[kanal_durchlauf] <= 0x87))    /* Wert zwischen 1.15 V und 1.24 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0xF3; 						                    /* Timer 0 Register fr berlauf nach 0.9ms laden */			 
	    TH0= 0xF1;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[2]= 0x12BB;		                /* Kanal-Pulse mit 1.2 ms fr Start-Pulse-Berechnung laden */
    }

	else if((analog_kanal[kanal_durchlauf] >= 0x88 ) && (analog_kanal[kanal_durchlauf] <= 0x92)) /* Wert zwischen 1.25 V und 1.34 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x63; 						                    /* Timer 0 Register fr berlauf nach 1.0ms laden */			 
	    TH0= 0xF0;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[3]= 0x144B;		                /* Kanal-Pulse mit 1.3 ms fr Start-Pulse-Berechnung laden */
    }								    

	else if((analog_kanal[kanal_durchlauf] >= 0x93 ) && (analog_kanal[kanal_durchlauf] <= 0x9D)) /* Wert zwischen 1.35 V und 1.44 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0xD3; 						                /* Timer 0 Register fr berlauf nach 1.1ms laden */			 
	    TH0= 0xEE;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[4]= 0x15DB;		                /* Kanal-Pulse mit 1.4 ms fr Start-Pulse-Berechnung laden */
    }

	else if((analog_kanal[kanal_durchlauf] >= 0x9E ) && (analog_kanal[kanal_durchlauf] <= 0xA7)) /* Wert zwischen 1.45 V und 1.54 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x44; 						                    /* Timer 0 Register fr berlauf nach 1.2ms laden */			 
	    TH0= 0xED;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[5]= 0x176A;                      /* Kanal-Pulse mit 1.5 ms fr Start-Pulse-Berechnung laden */
    }

	else if((analog_kanal[kanal_durchlauf] >= 0xA8 ) && (analog_kanal[kanal_durchlauf] <= 0xB2)) /* Wert zwischen 1.55 V und 1.64 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0xB4; 						                    /* Timer 0 Register fr berlauf nach 1.3ms laden */			 
	    TH0= 0xEB;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[6]= 0x18FA;		                /* Kanal-Pulse mit 1.6 ms fr Start-Pulse-Berechnung laden */
    }
	
	else if((analog_kanal[kanal_durchlauf] >= 0xB3 ) && (analog_kanal[kanal_durchlauf] <= 0xBD))    /* Wert zwischen 1.65 V und 1.74 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x25; 						                    /* Timer 0 Register fr berlauf nach 1.4ms laden */			 
	    TH0= 0xEA;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[7]= 0x1A89;		                /* Kanal-Pulse mit 1.7 ms fr Start-Pulse-Berechnung laden */
    }

	else if((analog_kanal[kanal_durchlauf] >= 0xBE ) && (analog_kanal[kanal_durchlauf] <= 0xC8))    /* Wert zwischen 1.75 V und 1.84 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x95; 						                    /* Timer 0 Register fr berlauf nach 1.5ms laden */			 
	    TH0= 0xE8;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[8]= 0x1C19;		                /* Kanal-Pulse mit 1.8 ms fr Start-Pulse-Berechnung laden */
    }

	else if((analog_kanal[kanal_durchlauf] >= 0xC9 ) && (analog_kanal[kanal_durchlauf] <= 0xD3))    /* Wert zwischen 1.85 V und 1.94 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x05; 						                    /* Timer 0 Register fr berlauf nach 1.6ms laden */			 
	    TH0= 0xE7;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[9]= 0x1DA9;		                /* Kanal-Pulse mit 1.9 ms fr Start-Pulse-Berechnung laden */
    }

	else if(analog_kanal[kanal_durchlauf] >= 0xD4 )               /* Wert grsser gleich 1.95 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x76; 						                    /* Timer 0 Register fr berlauf nach 1.7ms laden */			 
	    TH0= 0xE5;
        warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					                    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				                    /* Warteschlaufe bis berlauf stattfindet */
		return kanal_pulse1[10]= 0x1F38;		                /* Kanal-Pulse mit 2.0 ms fr Start-Pulse-Berechnung laden */
    }
}    


/***********************************************************************/
/* Funktion			: pulse_auswertung_kanal_2                     	   */
/* Funktion Input	: unsigned char			                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

unsigned int pulse_auswertung_kanal_2(unsigned char analog_kanal[], unsigned char kanal_durchlauf)
{ 
    
    unsigned int gier_min= 0x0000;              /* Zhler fr Gier-Dauer */    
    bit flag_gier_mitte= 0;                     /* Flag fr Gier Mitte */
    bit flag_gier_min= 0;                       /* Flag fr Gier Min */
    bit flag_gier_max= 0;                       /* Flag fr Gier Max */
    unsigned int gier_durchlauf= 1000;          /* Durchlauf fr Gier, ca. 2 Sek. */
    unsigned int kanal_pulse2[5];
    
    if((analog_kanal[kanal_durchlauf] >= 0x57 )  && (analog_kanal[kanal_durchlauf] <= 0xD9) && (flag_gier_mitte==0)) /* Wert zwischen 0.8 V und 2.0 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
        TL0= 0x44; 						  /* Timer 0 Register fr berlauf nach 1.2ms laden */			 
	    TH0= 0xED;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse2[0]= 0x176A;    /* Kanal-Pulse mit 1.5 ms fr Start-Pulse-Berechnung laden */
    }
		
    else if(analog_kanal[kanal_durchlauf] <= 0x56)      /* Wert kleiner gleich 1.19 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
		TL0= 0x63; 						  /* Timer 0 Register fr berlauf nach 1.0ms laden */			 
	    TH0= 0xF0;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        flag_gier_mitte= 1;               /* Flag Gier-Mitte setzen */
        flag_gier_min= 1;                 /* Flag Gier-Min setzen */
        return kanal_pulse2[1]= 0x144B;	  /* Kanal-Pulse mit 1.3 ms fr Start-Pulse-Berechnung laden */
	}
		
    else if(flag_gier_min==1)             /* Abfrage nur fr Kanal 2, Wert kleiner gleich 1.19 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
	    TL0= 0x63; 						  /* Timer 0 Register fr berlauf nach 1.0ms laden */			 
	    TH0= 0xF0;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        gier_durchlauf--;
        return kanal_pulse2[2]= 0x144B;	  /* Kanal-Pulse mit 1.3 ms fr Start-Pulse-Berechnung laden */
        if(gier_durchlauf==0)
        {                                  
            flag_gier_min= 0;             /* Flag Gier-Min lschen */
            flag_gier_mitte= 0;           /* Flag Gier-Mitte lschen */
            gier_durchlauf= 1000;         /* Gier_Durchlauf wieder laden */
        }
    }

	else if(analog_kanal[kanal_durchlauf] >= 0xDA )     /* Abfrage nur fr Kanal 2, Wert kleiner gleich 1.61 V bei VAREF 2.35 V / 256 = 0.0092V */
	{
		TL0= 0x25; 						  /* Timer 0 Register fr berlauf nach 1.4ms laden */			 
	    TH0= 0xEA;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        flag_gier_mitte= 1;               /* Flag Gier-Mitte setzen */
        flag_gier_max= 1;                 /* Flag Gier-Max setzen */
        return kanal_pulse2[3]= 0x1A89;	  /* Kanal-Pulse mit 1.7 ms fr Start-Pulse-Berechnung laden */
	}

    else if(flag_gier_max==1)             /* Abfrage nur fr Kanal 2, Wert kleiner gleich 1.61 V bei VAREF 2.35 V / 256 = 0.0092V */
	{
		TL0= 0x25; 						  /* Timer 0 Register fr berlauf nach 1.4ms laden */			 
	    TH0= 0xEA;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        gier_durchlauf--;
        return kanal_pulse2[4]= 0x1A89;	  /* Kanal-Pulse mit 1.7 ms fr Start-Pulse-Berechnung laden */
        if(gier_durchlauf==0)
        {
            flag_gier_max= 0;             /* Flag Gier-Min lschen */
            flag_gier_mitte= 0;           /* Flag Gier-Mitte lschen */
            gier_durchlauf= 1000;         /* Gier_Durchlauf wieder laden */
        }
	}
}


/***********************************************************************/
/* Funktion			: pulse_auswertung_kanal_4                     	   */
/* Funktion Input	: unsigned char			                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

unsigned int pulse_auswertung_kanal_4(unsigned char analog_kanal[], unsigned char kanal_durchlauf)
{
    
    unsigned int kamera_min= 0x0000;              /* Zhler fr Gier-Dauer */    
    bit flag_kamera_mitte= 0;                     /* Flag fr Gier Mitte */
    bit flag_kamera_min= 0;                       /* Flag fr Gier Min */
    bit flag_kamera_max= 0;                       /* Flag fr Gier Max */
    unsigned int kamera_durchlauf= 1000;          /* Durchlauf fr Gier, ca. 2 Sek. */
    unsigned int kanal_pulse4[5];
    
    if((analog_kanal[kanal_durchlauf] >= 0x67 )  && (analog_kanal[kanal_durchlauf] <= 0xEF) && (flag_kamera_mitte==0)) /* Wert zwischen 1.0 V und 2.2 V bei VAREF 2.35 V / 256 = 0.0092V */
    {
	    TL0= 0x44; 						  /* Timer 0 Register fr berlauf nach 1.2ms laden */			 
	    TH0= 0xED;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse4[0]= 0x176A;       /* Kanal-Pulse mit 1.5 ms fr Start-Pulse-Berechnung laden */
	}
		
    else if(analog_kanal[kanal_durchlauf] <= 0x62)      /* Wert kleiner gleich 0.9 V bei VAREF 2.35 V / 256 = 0.0092V */
	{
	    TL0= 0x63; 						  /* Timer 0 Register fr berlauf nach 1.0ms laden */			 
	    TH0= 0xF0;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        flag_kamera_mitte= 1;               /* Flag Gier-Mitte setzen */
        flag_kamera_min= 1;                 /* Flag Gier-Min setzen */
        return kanal_pulse4[1]= 0x144B;	  /* Kanal-Pulse mit 1.3 ms fr Start-Pulse-Berechnung laden */
	}
		
    else if(flag_kamera_min==1)             /* Ist Flag gesetzt */
	{
		TL0= 0x63; 						  /* Timer 0 Register fr berlauf nach 1.0ms laden */			 
	    TH0= 0xF0;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        kamera_durchlauf--;
        return kanal_pulse4[2]= 0x144B;	  /* Kanal-Pulse mit 1.3 ms fr Start-Pulse-Berechnung laden */
        if(kamera_durchlauf==0)
        {                                  
            flag_kamera_min= 0;             /* Flag Gier-Min lschen */
            flag_kamera_mitte= 0;           /* Flag Gier-Mitte lschen */
            kamera_durchlauf= 1000;         /* Gier_Durchlauf wieder laden */
        }
    }

	else if(analog_kanal[kanal_durchlauf] >= 0xFA)      /* Wert kleiner gleich 1.61 V bei VAREF 2.35 V / 256 = 0.0092V */
	{
		TL0= 0x25; 						  /* Timer 0 Register fr berlauf nach 1.4ms laden */			 
	    TH0= 0xEA;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        flag_kamera_mitte= 1;               /* Flag Gier-Mitte setzen */
        flag_kamera_max= 1;                 /* Flag Gier-Max setzen */
        return kanal_pulse4[3]= 0x1A89;	  /* Kanal-Pulse mit 1.7 ms fr Start-Pulse-Berechnung laden */
	}

    else if(flag_kamera_max==1)             /* Ist Flag gesetzt */
	{
		TL0= 0x25; 						  /* Timer 0 Register fr berlauf nach 1.4ms laden */			 
	    TH0= 0xEA;
        warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					  /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				  /* Warteschlaufe bis berlauf stattfindet */
        kamera_durchlauf--;
        return kanal_pulse4[4]= 0x1A89;	  /* Kanal-Pulse mit 1.7 ms fr Start-Pulse-Berechnung laden */
        if(kamera_durchlauf==0)
        {
            flag_kamera_max= 0;             /* Flag Gier-Min lschen */
            flag_kamera_mitte= 0;           /* Flag Gier-Mitte lschen */
            kamera_durchlauf= 1000;         /* Gier_Durchlauf wieder laden */
        }
	}
}


/***********************************************************************/
/* Funktion			: pulse_auswertung_kanal_5                     	   */
/* Funktion Input	: unsigned char			                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

unsigned int pulse_auswertung_kanal_5(void)
{
    unsigned int kanal_pulse5[2];
    
    if(P2_1 == 0)   	                    /* Abfrage Pin P0.1 gelscht */
	{
		TL0= 0x12; 						    /* Timer 0 Register fr berlauf nach 0.7ms laden */			 
	    TH0= 0xF5;
        warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse5[0]= 0x0F9C;		/* Kanal-Pulse mit 1.0 ms fr Start-Pulse-Berechnung laden */
	}
		   
	else if(P2_1 == 1)	                    /* Abfrage Pin P0.1 gesetzt */     
	{
        TL0= 0x76; 						    /* Timer 0 Register fr berlauf nach 1.7ms laden */			 
	    TH0= 0xE5;
        warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse5[1]= 0x1F38;		/* Kanal-Pulse mit 2.0 ms fr Start-Pulse-Berechnung laden */
    }    
}


/***********************************************************************/
/* Funktion			: pulse_auswertung_kanal_6                     	   */
/* Funktion Input	: unsigned char			                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

unsigned int pulse_auswertung_kanal_6(void)
{
    unsigned int kanal_pulse6[2];
    
    if(P2_2 == 0)   		                /* Abfrage Pin P0.1 gelscht */
	{
		TL0= 0x12; 						    /* Timer 0 Register fr berlauf nach 0.7ms laden */			 
	    TH0= 0xF5;
        warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse6[0]= 0x0F9C;		/* Kanal-Pulse mit 1.0 ms fr Start-Pulse-Berechnung laden */
	}
		   
	else if(P2_2 == 1)	                    /* Abfrage Pin P0.1 gesetzt */     
	{
        TL0= 0x76; 						    /* Timer 0 Register fr berlauf nach 1.7ms laden */			 
	    TH0= 0xE5;
        warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse6[1]= 0x1F38;		/* Kanal-Pulse mit 2.0 ms fr Start-Pulse-Berechnung laden */
    }
}


/***********************************************************************/
/* Funktion			: pulse_auswertung_kanal_7                     	   */
/* Funktion Input	: unsigned char			                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

unsigned int pulse_auswertung_kanal_7(void)
{
    unsigned int kanal_pulse7[2];
    
    if(P2_3 == 0)   		                        /* Abfrage Pin P0.1 gelscht */
    {
	    TL0= 0x12; 						    /* Timer 0 Register fr berlauf nach 0.7ms laden */			 
	    TH0= 0xF5;
        warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse7[0]= 0x0F9C;		/* Kanal-Pulse mit 1.0 ms fr Start-Pulse-Berechnung laden */
	}
		   
	else if(P2_3 == 1)	                    /* Abfrage Pin P0.1 gesetzt */     
	{
        TL0= 0x76; 						    /* Timer 0 Register fr berlauf nach 1.7ms laden */			 
	    TH0= 0xE5;
        warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
	    pulse_pause();					    /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
	    warte_ueberlauf();				    /* Warteschlaufe bis berlauf stattfindet */
        return kanal_pulse7[1]= 0x1F38;		/* Kanal-Pulse mit 2.0 ms fr Start-Pulse-Berechnung laden */
    }
}


/***********************************************************************/
/* Funktion			: start_pulse_auswertung                     	   */
/* Funktion Input	: unsigned char			                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

void start_pulse_auswertung(unsigned int kanal_pulse[], unsigned char anz_kanaele)
{
    
    unsigned long ppm_frame= 0x00015F36;    	/* Gesamter PPM-Frame = 22.5ms -> 89910 -> 0x00015F36 */
    unsigned int start_pulse= 0x0000; 			/* Start-Pulse fr PPM-Signal */
    unsigned int summe_kanal_pulse= 0x0000;     /* Variable um Summe aller einzelnen Kanle auszurechnen */         
    unsigned char timer_durchlauf= 0;           /* Variable um Durchlufe des Timer 0 zu zhlen bis richtige Zeit abgelaufen ist */
    unsigned char s;   		                    /* Index for-Schleife		*/
    

    for(s=0; s < anz_kanaele; s++)                                /* Schleife um die Summe der Kanalimpulse zu eruieren und zu berechnen */
    {
        summe_kanal_pulse += kanal_pulse[s];     /* Addieren von den einzelnen 8 Kanalimpulse fr den Start-Pulse */
    }                                             
     
	
    start_pulse= ppm_frame - summe_kanal_pulse;             /* Start-Pulse berechnen */
    timer_durchlauf= start_pulse / 0xFF;                    /* Durchlufe berechnen bis im Timer 0 richtige Zeit abgelaufen ist */          
    flag_reload= 1;                                         /* Flag-Reload setzen um in der Timer 0 ISR die Register wieder neu zu laden */
    P2_0= 1;
    TL0= 0x00;                                              /* Timer 0 Register mit Wert 65280 laden -> alle 256 Schritte ein berlauf */
    TH0= 0xFF;
    while(t != timer_durchlauf);                             /* Warteschlaufe bis berlauf nach stattfindet */
    flag_reload= 0;                                         /* Flag-Reload wieder lschen */
    pulse_pause();					                        /* Timer 0 Register fr Pulse_Pause von 0.3ms laden */
    warte_ueberlauf();				                        /* Warteschlaufe bis berlauf stattfindet */
                     
}







/***********************************************************************/
/* Funktion			: init_adc ISR -> Initialisierung ADC          	   */
/* Funktion Input	: void					                           */
/* Funktion Output  : void		     	                               */
/***********************************************************************/

void init_adc(void)
{
    ADCF= 0xFF;								  /* Kanal 0 bis 7 whlen	         */
	
    ADCLK= 0x00;							  /* Takt des ADC's whlen         */
											  /* fADC = fcpu clock/ (4 (or 2 in X2 mode)* PRS */
											  /* Wenn PRS= 0, dann 24 MHz / (2 * 32) = 375 kHz */
											
    ADCON= 0x20;       		               	  /* ADC einschalten 		 */

    EA = 1;                             	  /* Global Interrupts einschalten */
    EADC = 1;                          		  /* ADC Interrupt einschalten     */
}	    


/***********************************************************************/
/* Funktion			: ADC ISR -> Speichert umgewandelter Wert      	   */
/* Funktion Input	: void					       					   */
/* Funktion Output  : void		                               	       */
/***********************************************************************/

void is_Adc(void) interrupt 8
{
    ADCON &= ~0x10;                      	  /* End of conversion IR flag lschen */
    umgewandelter_wert= ADDH;           	  /* Analogwert speichern */
    ende_umwandlung=1;                 		  /* Software flag setzen, um mit Programm fortzufahren */
}	

	
/***********************************************************************/
/* Funktion			: init_timer0 -> Initialisierung Timer 0       	   */
/* Funktion Input	: void			                       			   */
/* Funktion Output  : void		                                       */
/***********************************************************************/

void init_timer0(void)
{  
    TMOD = (TMOD & 0xF0) | 0x01;  			  /* Einstellung Timer 0 / Mode 1 -> 16 Bit */
    ET0 = 1;                      			  /* Timer 0 Interrupts einschalten*/
    TR0 = 1;                      			  /* Start Timer 0 - Betrieb */
}	

	
/***********************************************************************/
/* Funktion			: Timer 0 ISR -> Initialisierung Timer 0       	   */
/* Funktion Input	: void			                       			   */
/* Funktion Output  : void		                                	   */
/***********************************************************************/
	
void timer0_ISR(void) interrupt 1
{
    if(flag_reload==0)                              /* Abfrage ob Flag-Reload nicht gesetzt ist */
    {
        P2_0= ~P2_0;						      /* Invertiere Pin 2.0 fr PPM-Signal */
        flag_ueberlauf= 1;						  /* flag_berlauf setzen, um mit Programm fortzufahren*/
    }	
    else if(flag_reload==1)                          /* Abfrage ob Flag-Reload gesetzt ist */
    {
        TL0= 0x00;                                /* Timer 0 Register mit Wert 65280 laden -> alle 256 Schritte ein berlauf */
        TH0= 0xFF;
        t++;                                      /* t inkrementieren bis t gleich durchlauf, somit gewnschte Start-Puls erreicht */
        if(t==timer_durchlauf)
        {
            P2_0= ~P2_0;						  /* Invertiere Pin 2.0 fr PPM-Signal */
        }         
    }
}

/***********************************************************************/
/* Funktion			: pulse_pause -> Definiert Pulspause von 0.3ms 	   */
/* Funktion Input	: void			                       			   */
/* Funktion Output  : void		     	                         	   */
/***********************************************************************/	

void pulse_pause(void)
{
    TL0= 0x50;								  /* Timer 0 Register fr berlauf nach 0.3ms laden */						
    TH0= 0xFB;	
}	


/***********************************************************************/
/* Funktion			: warte_berlauf -> Warteschlaufe bis      		   */
/*   			  	  berlauf stattfindet                    		   */
/* Funktion Input	: void			                       			   */
/* Funktion Output  : void		                      	         	   */
/***********************************************************************/	

void warte_ueberlauf(void)
{
    while(!flag_ueberlauf);				     /* Warten bis berlauf stattfindet */
    flag_ueberlauf= 0;						 /* flag_berlauf wieder lschen */
}		
						   

/***********************************************************************/
/* Funktion			: wait_speaker -> Definiert Pulspause von 0.3ms    */
/* Funktion Input	: void			                       			   */
/* Funktion Output  : void		     	                         	   */
/***********************************************************************/	

void wait_speaker(void)
{
    unsigned int l;
	for(l=0; l<=30000; l++);
    P0 ^= 0xC0;	                                /* Invertiere Pin P0.6 (rote LED) und P0.7 (Speaker) */
}	
